
import * as _ from 'lodash'

import {
    HttpHeaders,
    HttpQueryParams,
    NativeFetchOptions,
} from '../index.d'

class NativeFetch {
    private baseUrl: string
    private globalHeaders: HttpHeaders = {}

    constructor(baseUrl: string, options: NativeFetchOptions = { request: { contentType: 'application/json' } }) {
        this.baseUrl = baseUrl
        if (options.request && options.request.contentType) {
            this.setGlobalHeader('Content-Type', options.request.contentType)
        }
    }

    private static StringifyQueryParams = (queryParams: HttpQueryParams): string => {
        const params = []
        for (const key in queryParams) {
            params.push(`${key}=${encodeURIComponent(String(queryParams[key]))}`)
        }
        if (params.length > 0) {
            return `?${params.join('&')}`
        }
        return ''
    }

    public setGlobalHeader = (name: string, value: string): void => { this.globalHeaders[name] = value }
    public removeGlobalHeader = (name: string): void => { delete this.globalHeaders[name] }

    public httpGet = async <Response>(path: string, queryParams: HttpQueryParams, headers: HttpHeaders = {}): Promise<Response> => {
        const response = await fetch(`${this.baseUrl}${path}${NativeFetch.StringifyQueryParams(queryParams)}`, {
            method: 'GET',
            headers: _.merge({}, this.globalHeaders, headers),
        })
        return await response.json()
    }

    public httpPost = async <Response>(path: string, body: any, headers: HttpHeaders = {}): Promise<Response> => {
        const response = await fetch(`${this.baseUrl}${path}`, {
            body, method: 'POST',
            headers: _.merge({}, this.globalHeaders, headers),
        })
        return await response.json()
    }
}

export { NativeFetch }
